/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *
 *  Copyright (C) 1999 by the Bahamut Development Team.
 *  Copyright (C) 2011 by the Hybrid Development Team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 */

#include "stdinc.h"
#include "client.h"
#include "ircd.h"
#include "s_serv.h"
#include "send.h"
#include "channel_mode.h"
#include "parse.h"
#include "modules.h"
#include "irc_string.h"
#include "s_user.h"
#include "conf.h"


/*! \brief AUTHFLAGS command handler (called by services)
 *
 * \param client_p Pointer to allocated Client struct with physical connection
 *                 to this server, i.e. with an open socket connected.
 * \param source_p Pointer to allocated Client struct from which the message
 *                 originally comes from.  This can be a local or remote client.
 * \param parc     Integer holding the number of supplied arguments.
 * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
 *                 pointers.
 */
static void
me_authflags(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
{
  struct Client *target_p;

  if (!IsServer(source_p))
    return;

  target_p = find_person(client_p, parv[1]);
  if (target_p == NULL)
    return;

  for (size_t i = 0; i < strlen(parv[2]); ++i)
  {
    switch (parv[2][i])
    {
      case '$':
        SetExemptResv(target_p);
        break;
      case '=':
        SetIPSpoof(target_p);
        break;
      case '^':
        SetExemptKline(target_p);
        break;
      case '_':
        SetExemptGline(target_p);
        break;
      case '>':
        SetExemptLimits(target_p);
        break;
      case '|':
        SetCanFlood(target_p);
        break;
    }
  }
}

static struct Message authflags_msgtab = {
  .cmd = "AUTHFLAGS",
  .args_min = 3,
  .args_max = MAXPARA,
  .handlers[UNREGISTERED_HANDLER] = m_ignore,
  .handlers[CLIENT_HANDLER] = m_ignore,
  .handlers[SERVER_HANDLER] = m_ignore,
  .handlers[ENCAP_HANDLER] = me_authflags,
  .handlers[OPER_HANDLER] = m_ignore,
};

static void
module_init(void)
{
  mod_add_cmd(&authflags_msgtab);
}

static void
module_exit(void)
{
  mod_del_cmd(&authflags_msgtab);
}

struct module module_entry = {
  .version = "$Revision$",
  .modinit = module_init,
  .modexit = module_exit,
};
